package loganalysis;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.StreamTokenizer;
import java.util.HashMap;
import java.util.Iterator;

import org.apache.commons.collections4.keyvalue.MultiKey;



public class LogAnalyzer {

	//String path = "./features-dartmouth/";
	String path = ".";
	String name;
	
	HashMap<Integer, NodeFeatures> nodesFeatures = new HashMap<Integer, NodeFeatures>();
	
	HashMap<MultiKey<Integer>, EdgeFeatures> edgesFeatures = new HashMap<MultiKey<Integer>, EdgeFeatures>();
	
	
	public void writeNodesSummary(String name) {
		try {
			File file = new File("summaryNodes" + name + ".dat");
			FileWriter writer = new FileWriter(file);
			PrintWriter out = new PrintWriter(writer);
			
			Iterator<Integer> nodesIds = nodesFeatures.keySet().iterator();
			
			while(nodesIds.hasNext()) {
				int nodeId = nodesIds.next();
				NodeFeatures nf = nodesFeatures.get(nodeId);
				
				out.println(nf.id + " " + nf.degree.getMean() + " " + nf.degree.getStandardDeviation() + " " + nf.degree.getSkewness() + " " +
						nf.closeness.getMean() + " " + nf.closeness.getStandardDeviation() + " " + nf.closeness.getSkewness() + " " +
						nf.betweeness.getMean() + " " + nf.betweeness.getStandardDeviation() + " " + nf.betweeness.getSkewness() + " " +
						nf.pageRank.getMean() + " " + nf.pageRank.getStandardDeviation() + " " + nf.pageRank.getSkewness());
			}
			
		} catch(Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}
	
	public void writeEdgesSummary(String name) {
		try {
			File file = new File("summaryEdges" + name + ".dat");
			FileWriter writer = new FileWriter(file);
			PrintWriter out = new PrintWriter(writer);
			
			Iterator<MultiKey<Integer>> edgesIds = edgesFeatures.keySet().iterator();
			
			while(edgesIds.hasNext()) {
				MultiKey<Integer> edgeId = edgesIds.next();
				EdgeFeatures ef = edgesFeatures.get(edgeId);
				
				out.println(ef.edgeKey.getKey(0) + " " + ef.edgeKey.getKey(1) + " " +
						ef.persistency.getMean() + " " + ef.persistency.getStandardDeviation() + " " + ef.persistency.getSkewness() + " " +
						ef.beetweeness.getMean() + " " + ef.beetweeness.getStandardDeviation() + " " + ef.beetweeness.getSkewness() + " " +
						ef.weight.getMean() + " " + ef.weight.getStandardDeviation() + " " + ef.weight.getSkewness());
			}
			
		} catch(Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}
		
	
	public void processEdgeFile(File file) {
		
		try {
            //inicializa o StreamTokenizer
            FileReader reader = new FileReader(file);
            BufferedReader bReader = new BufferedReader(reader);
            StreamTokenizer st = new StreamTokenizer(bReader);   

            //configura o StreamTokenizer
            st.resetSyntax();
            st.whitespaceChars(0,' ');
            st.wordChars(' '+1,255);
           
            //manda o StreamTokenizer ler a primeira palavra do arquivo
            st.nextToken();
            //pega a linha atual do arquivo

            while(st.ttype!=StreamTokenizer.TT_EOF){
                int v1 = Integer.parseInt(st.sval);
                st.nextToken();
                int v2 = Integer.parseInt(st.sval);
                MultiKey<Integer> edgeKey = new MultiKey<Integer>(v1, v2);
                if(!edgesFeatures.containsKey(edgeKey)) {
                	EdgeFeatures ef = new EdgeFeatures(edgeKey);
                	edgesFeatures.put(edgeKey, ef);
                }
                
                EdgeFeatures ef = edgesFeatures.get(edgeKey);
                
                st.nextToken();
                double beetweeness= Double.parseDouble(st.sval);
                ef.addBeetweeness(beetweeness);
                
                st.nextToken();
                double persistency = Double.parseDouble(st.sval);
                ef.addPersistency(persistency);
                
                st.nextToken();
                double weight = Double.parseDouble(st.sval);
                ef.addWeight(weight);

                st.nextToken();
            }
           
            bReader.close();
            reader.close();

        } catch(Exception e) {
            System.out.println("Erro!");
            e.printStackTrace();
        }		
	}	
	
	public void processNodeFile(File file) {
		
		try {
            //inicializa o StreamTokenizer
            FileReader reader = new FileReader(file);
            BufferedReader bReader = new BufferedReader(reader);
            StreamTokenizer st = new StreamTokenizer(bReader);   

            //configura o StreamTokenizer
            st.resetSyntax();
            st.whitespaceChars(0,' ');
            st.wordChars(' '+1,255);
           
            //manda o StreamTokenizer ler a primeira palavra do arquivo
            st.nextToken();
            //pega a linha atual do arquivo
            int linhaAtual = st.lineno();
            //enquanto não for fim de arquivo, leia mais palavras
            st.nextToken();
            st.nextToken();
            st.nextToken();
            while(st.ttype!=StreamTokenizer.TT_EOF){
                int id = Integer.parseInt(st.sval);
                if(!nodesFeatures.containsKey(id)) {
                	NodeFeatures nf = new NodeFeatures(id);
                	nodesFeatures.put(id, nf);
                }
                
                NodeFeatures nf = nodesFeatures.get(id);
                
                st.nextToken();
                double closeness = Double.parseDouble(st.sval);
                nf.addCloseness(closeness);
                st.nextToken();
                double degree = Double.parseDouble(st.sval);
                nf.addDegree(degree);
                st.nextToken();
                double betweeness= Double.parseDouble(st.sval);
                nf.addBetweeness(betweeness);
                st.nextToken();
                double pageRank = Double.parseDouble(st.sval);
                nf.addPageRank(pageRank);
                if(id == 896)
                	System.out.println(id);
                st.nextToken();
            }
           
            bReader.close();
            reader.close();

        } catch(Exception e) {
            System.out.println("Erro!");
            e.printStackTrace();
        }		
	}
	
	public void readNodeLogs(String pattern) {
		
		
		try {
			String fileName;
			File folder = new File(path);
			File[] listOfFiles = folder.listFiles();
			for (int i = 0; i < listOfFiles.length; i++) {
				if (listOfFiles[i].isFile()) {
					fileName = listOfFiles[i].getName();
					if (fileName.contains(pattern)) {
						System.out.println(fileName);
						if(fileName.contains("nodes") && fileName.contains("features"))
							processNodeFile(listOfFiles[i]);
						else if(fileName.contains("edges") && fileName.contains("features"))
							processEdgeFile(listOfFiles[i]);
					}
				}
			}
			writeNodesSummary(pattern);
			
		} catch(Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}
	
	
	

	public static void main(String[] args) {
		LogAnalyzer la = new LogAnalyzer();
		la.readNodeLogs("nodes2-Windowed");
	}
}
