package main;

import scheduler.Scheduler;
import scheduler.SchedulerNus;
import scheduler.SchedulerOneFile;
import scheduler.SchedulerSassy;
import scheduler.SchedulerSeveralFiles;
import scheduler.SchedulerTestSet;
import scheduler.SchedulerUPB;

public class DB {
	
	public final static int NUM_CLASSES = 4;
	
	public final static int DB_DARTMOUTH = 0;
	public final static int DB_CAMBRIDGE = 1;
	public final static int DB_TAXI = 2;
	public final static int DB_INFOCOM = 3;
	public final static int DB_USC = 4;
	public final static int DB_NUS = 5;
	public final static int DB_UPB = 6;
	public final static int DB_SASSY = 7;
	public final static int DB_4SQUARE_NY = 8;
	public final static int DB_4SQUARE_TK = 9;
	public final static int DB_4SQUARE_SP = 10;
	
	public final static String DB_FOLDER = "..\\traces\\";
	//public final static String DB_FOLDER = "D:\\INRIA\\INRIA TRACES\\";
	
	public static String[] DB_NAME = {
		"Dart",
		"Cambridge",
		"Taxi",
		"Infocom",
		"USC",
		"NUS",
		"UPB",
		"SASSY",
		"4SQUARE-NY",
		"4SQUARE-TK",
		"4SQUARE-SP"};
	
	
	public static String[] DB_PATH = {
		DB_FOLDER + "dartmouthsorted.trace",
		DB_FOLDER + "HAGGLE-CAMBRIDGE/trace-cambridge-longtraining20-500-0.0134-099-1.45-1day.converted.dat",
		DB_FOLDER + "EPFL-TAXI\\FROM_ALINE\\Taxi-using-250m-range",
		DB_FOLDER + "INFOCOM_EXPANDED/FROM_ALINE/780",
		DB_FOLDER + "USC\\ContactsUSC.dat",
		DB_FOLDER + "NUS\\mobicom06-trace.txt",
		DB_FOLDER + "UPB\\interactions.dat",
		DB_FOLDER + "SASSY\\dsn.csv",
		DB_FOLDER + "4SQUARE-NY\\foursquare-all-NY.encounters",
		DB_FOLDER + "4SQUARE-TK\\foursquare-all-TK.encounters",
		DB_FOLDER + "4SQUARE-SP\\foursquare-all-SP.encounters"};
	
	public static String[] DB_GRAPHS_PATH = {
		"../Graphs_Data/Dart/",
		"../Graphs_Data/Cambridge/",
		"../Graphs_Data/Taxi/",
		"../Graphs_Data/Infocom/",
		"../Graphs_Data/USC/",
		"../Graphs_Data/NUS/",
		"../Graphs_Data/UPB/",
		"../Graphs_Data/SASSY/",
		"../Graphs_Data/4SQUARE-NY/",
		"../Graphs_Data/4SQUARE-TK/",
		"../Graphs_Data/4SQUARE-SP/"};	
	
	public static String[] DB_FEATURES_PATH = {
		"./features-dartmouth/",
		"./features-cambridge/",
		"./features-taxi/",
    	"./features-infocom/",
		"./features-USC/"};	

	public static String[] DB_PATH_RANDOMEDGES = {
		"/home/pedro/dtnnc/trunk/DTN_SNA/SNA/features-dartmouth/24hours/edges/edges24-Aggregated1077976994.features",
		null,
		null,
		null,
		"/home/pedro/dtnnc/trunk/DTN_SNA/SNA/features-USC/edges/edges24-Aggregated5890006.features",
		null};
	
	public static int[] DB_NUM_NODES = {
		1146,
		3600*24,
		24,
		1,
		3600*24,
		1,
		3600*24,
		3600*24,
		0,
		0,
		0};	
	
	public static int[] DB_RECORD_INTERVAL = {
		3600*24,
		3600*24,
		24,
		1,
		3600*24,
		1,
		3600*24,
		3600*24,
		3600*24*7,
		3600*24*7,
		3600*24*7};
	
	public static int[] DB_WEIGHT_SCALE = {
		3600,
		3600,
		1,
		1,
		3600,
		1,
		3600,
		3600,
		3600,
		3600,
		3600};	
	
	public static long[] DB_TRAINING_UPPER_BOUND = { // times DB_RECORD_INTERVAL
		28,
		0,
		28,
		0,
		28, //42
		0,
		9999,
		9999};
	
	public static long[] DB_TEST_UPPER_BOUND = {
		42, //35,
		0,
		0,
		0,
		42, //62
		0,
		9999,
		9999};	
	
	public static long[] DB_INFECTION_DURATION = {
		604800,//302400,
		0,
		0,
		0,
		604800, //907200 , // 1.5 weeks
		0};		
	

	public static int myDB;
	
	public static String getDBName() {
		return DB_NAME[myDB];
	}
	
	public static String getNodeTableFile() { 
		return DB_FOLDER + DB.getDBName() + "\\" + DB.getDBName() + "_nodetable.dat";
	}
	
	public static String getDBTestFile() {
		return DB_FOLDER + DB.getDBName() + "\\" + DB.getDBName() + "_start" + DB.getTrainingSetUpperBound() + "_end" + DB.getTestSetUpperBound() +  "_test.dat";
	}
	
	public static String getFilePath() {
		if(RunConfig.RUN_OVER_TEST_SET)
			return DB.getDBTestFile();
		return DB_PATH[myDB];
	}
	
	public static String getFeaturesPath() {
		return DB_FEATURES_PATH[myDB];
	}
	
	public static String getGraphsPath() {
		return DB_GRAPHS_PATH[myDB];
	}	
	
	public static String getPathRandomEdges() {
		return DB_PATH_RANDOMEDGES[myDB];
	}
	
	public static Scheduler getScheduler() throws Exception {
		if(RunConfig.RUN_OVER_TEST_SET)
			return new SchedulerTestSet(DB.getFilePath());
		else if(myDB == DB_DARTMOUTH || myDB == DB_CAMBRIDGE || myDB == DB_USC || myDB == DB_4SQUARE_NY || myDB == DB_4SQUARE_TK || myDB == DB_4SQUARE_SP) {
			return new SchedulerOneFile(DB.getFilePath());
		}
		else if(myDB == DB_TAXI || myDB == DB_INFOCOM) {
			return new SchedulerSeveralFiles(DB.getFilePath());
		}
		else if(myDB == DB_NUS)
			return new SchedulerNus(DB.getFilePath());
		else if(myDB == DB_UPB)
			return new SchedulerUPB(DB.getFilePath());
		else if(myDB == DB_SASSY)
			return new SchedulerSassy(DB.getFilePath());		
		return null;
	}
	
	public static int getRecordInterval() {
		return DB_RECORD_INTERVAL[myDB];
	}
	
	public static int getWeightScale() {
		return DB_WEIGHT_SCALE[myDB];
	}	
	   
	public static long getTrainingSetUpperBound() {
		return DB_TRAINING_UPPER_BOUND[myDB];
	}
	
	public static long getTestSetUpperBound() {
		return DB_TEST_UPPER_BOUND[myDB];
	}	
	
	public static long getInfectionDuration() {
		return DB_INFECTION_DURATION[myDB];
	}
	 

}
