package main;

import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Random;
import java.util.Stack;

import org.apache.commons.collections15.keyvalue.MultiKey;
import org.apache.commons.math.stat.descriptive.DescriptiveStatistics;

import propagation.EveryoneInfectedException;
import propagation.PredictionByClass;
import propagation.PropagationSNA;
import edu.uci.ics.jung.graph.UndirectedSparseGraph;
import network.RandomNetworkGenerator;
import network.functions.GlobalFunctions;
import network.graph.Edge;
import network.graph.MyGraph;
import scheduler.Event;
import scheduler.NodeEvents;
import scheduler.Scheduler;
import scheduler.SchedulerOneFile;
import scheduler.SchedulerSeveralFiles;

public class Run {
	
	MyGraph g = new MyGraph("Dynamic"); //time evolving graph
	MyGraph G = new MyGraph("Aggregated"); //aggregated graph
	MyGraph Gt = new MyGraph("Windowed"); //aggregated graph per time step
	
	
	RandomNetworkGenerator randNet = new RandomNetworkGenerator();
	

	PropagationSNA propagation;
	PredictionByClass prediction;
	
	String name;
	
	
	public int AGI_NUM_HOURS;
	
	private long saveInfoTime = 0;
	
	long T0 = 0;
	private long time;
	private int timestep;
	
	Integer auxInt=0;
	long numChanges=0;
	long numChangesRand=0;
	
	long numEncounters = 0;
	
	Scheduler scheduler;

	
	public Run(Scheduler s, PropagationSNA propagation) throws Exception{
		
		g.COMPUTE_COMPONENTS_DYNAMICLY = false;
		this. AGI_NUM_HOURS = (int)Math.ceil((double)(DB.getRecordInterval()) / DB.getWeightScale());

		this.name = DB.getDBName();
        this.scheduler = s;
        
        this.propagation = propagation;
        propagation.setEventGraph(g);
        
        if(RunConfig.EXECUTE_PREDICTION)
        	prediction = new PredictionByClass();
        
        this.loadGraph();
	}
	

	public void printGlobalFeatures(MyGraph graph, long time) throws Exception {

		int numComponents = 0;
		double assortativity = 0;
		double ccG = 0;
		double ccRand = 0;
		double diameterRand = 0;
		double diameterG = 0;
		
		double changeRate = numChanges/(2.0*(time - T0 + 1)); //2 because we count the edge twice
		double changeRateRand = numChangesRand/(2.0*(time - T0 + 1)); //2 because we count the edge twice
		
		if(RunConfig.GF_RECORD_NUMBER_COMPONENTS)		
			numComponents = GlobalFunctions.getNumberOfComponents(graph);
		
		if(RunConfig.GF_RECORD_ASSORTATIVITY)
			assortativity = GlobalFunctions.getAssortativityDegree(graph);
		
		System.out.println("random comparison...");
		if(RunConfig.GF_RECORD_RANDOM_COMPARISON) {
			MyGraph gRand = RandomNetworkGenerator.generateERGraph3(graph);
			System.out.println("done...");

			ccRand = GlobalFunctions.getCC(gRand);
			diameterRand = 0;
			//GlobalFunctions.getDiameter(gRand);
			diameterRand = GlobalFunctions.getDiameter(gRand);
		}

		System.out.println("cc...");
		if(RunConfig.GF_RECORD_CC)
			ccG = GlobalFunctions.getCC(graph);
		System.out.println("done...");
		
		if(RunConfig.GF_RECORD_DIAMETER)
			diameterG = GlobalFunctions.getDiameter(graph);
		
		File file = new File("out" + AGI_NUM_HOURS + "-" + graph.getName() + "NORANDOM-10.txt");
		FileWriter writer = new FileWriter(file, true);
		PrintWriter out = new PrintWriter(writer);    	
		
		String output = "" + time + " " + changeRate + " " + changeRateRand + " " + numComponents + " " + assortativity + " "
		+ numChanges + " " + numChangesRand + " " + ccRand + " " + ccG + " " + diameterRand + " " + diameterG;
		
		System.out.println(output);
		out.println(output);
		
		out.close();
		writer.close();
	}
	

	
	
	
	public void takeSnapshot(long time, int timestep) throws Exception {
		timestep = timestep - 1;
		if(RunConfig.SAVE_GRAPH_FILE) {
			Gt.saveGraphToFile(Gt.getName() + AGI_NUM_HOURS + "-" + timestep + ".graph");
			G.saveEdgesFeaturesToFile(G.getName() + AGI_NUM_HOURS + "-" + timestep + ".graph");
			g.saveGraphToFile(g.getName() + AGI_NUM_HOURS + "-" + timestep + ".graph");
		}

}


	
	
	
	private void updateGraph(long time) throws Exception {

		if( (time - this.saveInfoTime) >= DB.getRecordInterval()) {
			
			numChangesRand += randNet.incrementRandomAgGraph(Gt);

			this.numChanges = 0;
			
			Gt.clear();

			this.saveInfoTime = time;
		}
	}

	
	
	
	
	public void saveInfo(long time) throws Exception {

		G.setTime(timestep);
		Gt.setTime(timestep);
		g.setTime(timestep);
		
		if( (time - this.saveInfoTime) >= DB.getRecordInterval()) {
			
			System.out.println("Timestep: " + timestep);
			takeSnapshot(time,timestep);
			updateGraph(time);

		}		
	}
	
	public void loadEvent(Event e) throws Exception{

		MyGraph gEvents = new MyGraph("gEvents");
		while(e.hasNext()) { 
			NodeEvents ne = e.getNext();
			this.time = e.getTime();
			
			if(RunConfig.LOAD_EVENT_G) {
				ArrayList<Edge> dlist = ne.dList.getEdgesL();
				for(int i=0;i<dlist.size(); i++)
					g.removeEdge(ne.id, dlist.get(i).getNodeDest());
			}
			
			ArrayList<Edge> clist = ne.cList.getEdgesL();
			for(int i=0;i<clist.size(); i++) {
				this.numEncounters++;
				
				Edge edge = clist.get(i);
				
				if(RunConfig.LOAD_EVENT_G)
					g.addEdge(ne.id, clist.get(i));
				if(RunConfig.LOAD_WINDOWED_G)
					Gt.addEdge(ne.id, clist.get(i));
				if(RunConfig.LOAD_FULL_G)
					this.numChanges += G.addEdge(ne.id, edge) == true ? 1 : 0;
					
			}
			
			if(RunConfig.EXECUTE_PROPAGATION || RunConfig.EXECUTE_PREDICTION) {
				for(int i=0;i<clist.size(); i++) {
					
					Edge edge = clist.get(i);
					try {
						if(RunConfig.EXECUTE_PROPAGATION)
							propagation.processContact(edge.getNodeSrc(), edge.getNodeDest(), e.getTime());
						if(RunConfig.EXECUTE_PREDICTION)
							prediction.processContact(edge.getNodeSrc(), edge.getNodeDest(), e.getTime());
					} catch(EveryoneInfectedException eie) {
						System.out.println("Everyone infected");
					}
				}
			}			
		}
		
		gEvents.clear();
		
	}	
	
	
	
	public void loadGraph() {
		try {
			int count = 0;
			while(this.scheduler.hasNextEvent()) {
				count++;
				Event e = this.scheduler.getNextEvent();
				
				if(count == 1) {
					this.T0 = e.getTime();
				}
				
				this.timestep = (int)Math.round((e.getTime()-this.T0)/DB.getRecordInterval())+1;
				
				this.loadEvent(e);
				this.saveInfo(e.getTime());

			}
			if(RunConfig.EXECUTE_PROPAGATION)
				propagation.finalizePropagation();
			if(RunConfig.EXECUTE_PREDICTION)
				prediction.saveClassCount();
			System.out.println(this.numEncounters);
			printGlobalFeatures(this.G, this.time);
			
			//propagation.printDelays();
			/*System.out.println(red.RANDOM_THRESH + " " + (double)this.socialEdgesThatAppeared.size()/this.socialEdges.size() + " " + 
					(double)this.randomEdgesThatAppeared.size()/this.randomEdges.size() + 
					" " + this.socialEdgesTime.getMean() + " " + this.socialEdgesTime.getStandardDeviation() + " " + " " + this.socialEdgesTime.getPercentile(50) + 
					" " + this.randomEdgesTime.getMean() + " " + this.randomEdgesTime.getStandardDeviation() + " " + " " + this.randomEdgesTime.getPercentile(50));*/
		} catch(Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}	
	}

	
	public static void main(String[] args) {
		
		try {
			
			//Run.runSeverePropagation();
			//Run.runSeverePropPreviousStartTimes();
			RunScripts.runGenerateGraphs(0);
			//Run.runMulticastPropagation();
			
		} catch(Exception e) {
			e.printStackTrace();
			System.out.println(e.getMessage());
		}


	}

}
