package network.functions;

import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Queue;

import org.apache.commons.math.stat.descriptive.DescriptiveStatistics;

import network.graph.Edge;
import network.graph.MyGraph;



public class DistanceFunctions {
	

	private int distances[][];
	MyGraph g;
	double avgDistance, maxDistance;
	HashMap<Integer, Double> nodeCloseness;
	
	public DistanceFunctions(MyGraph g) {
		this.g = g;
	}
	


	public int[][] getDistancesVector() {
		return distances;
	}
	
	public double getAvgDistance() {
		return this.avgDistance;
	}
	
	public double getMaxDistance() {
		return this.maxDistance;
	}
	
	public double getNodeCloseness(int nodeid) {
		return this.nodeCloseness.get(nodeid);
	}
	
	public void generateDistances() {
		Integer idNodes[] = g.getNodes();
		int n = idNodes.length;
		int maxId = g.getMaxId() + 1;
		nodeCloseness = new HashMap<Integer, Double>();
		distances = new int[maxId][maxId];
		for(int i=0; i<maxId; i++) 
			for(int j=0; j<maxId; j++) 
				distances[i][j] = Integer.MAX_VALUE;
		for(int i=0; i<n; i++) {
			this.generateDistances(idNodes[i]);
		}
		
		DescriptiveStatistics ds = new DescriptiveStatistics();
		DescriptiveStatistics dsNode = new DescriptiveStatistics();
		for(int i=0; i<n; i++) {
			dsNode.clear();
			for(int j=0; j<n; j++) {
				if(distances[idNodes[i]][idNodes[j]] != Integer.MAX_VALUE) {
					ds.addValue(this.distances[idNodes[i]][idNodes[j]]);
					dsNode.addValue(this.distances[idNodes[i]][idNodes[j]]);
				}
			}
			this.nodeCloseness.put(idNodes[i], 1/dsNode.getMean());
		}
		
		this.avgDistance = ds.getMean();
		this.maxDistance = ds.getMax();
		
		
	}		
	
	public void generateDistances(int node) {
		Queue<Integer> queue1 = new LinkedList<Integer>();
		Queue<Integer> queue2 = new LinkedList<Integer>();
		Integer idNodes[] = g.getNodes();
		int n = idNodes.length;

		
		HashSet<Integer> visited = new HashSet<Integer>();		
		int distance = 1;
		queue1.add(node);
		visited.add(node);
		int head = 0;
		while(!queue1.isEmpty() || !queue2.isEmpty()) {
			while(!queue1.isEmpty()) {
				head = queue1.poll();
				Edge[] edges = g.getEdges(head);
				for(int i=0; i<edges.length; i++) {
					int neigh = edges[i].getNodeDest();
					if(!visited.contains(neigh)) {
						if(distances[node][neigh] < Integer.MAX_VALUE) {
							for(int j=0; j<n; j++) {
								if(distances[neigh][idNodes[j]] < Integer.MAX_VALUE && node != idNodes[j] &&
										distances[node][idNodes[j]] > (distances[neigh][idNodes[j]] + distances[node][neigh])) {
									distances[node][idNodes[j]] = distances[neigh][idNodes[j]] + distances[node][neigh];
									distances[idNodes[j]][node] = distances[node][idNodes[j]];
								}
							}
						}
						else 
							queue2.add(neigh);
						visited.add(neigh);
						distances[neigh][node] = distance;
						distances[node][neigh] = distance;

					}
				}
			}
			distance++;
			
			while(!queue2.isEmpty()) {
				//System.out.println("Head of queue1 is" + head);
				head = queue2.poll();
				Edge[] edges = g.getEdges(head);
				for(int i=0; i<edges.length; i++) {
					int neigh = edges[i].getNodeDest();
					if(!visited.contains(neigh)) {
						if(distances[node][neigh] < Integer.MAX_VALUE) {
							for(int j=0; j<n; j++) {
								if(distances[neigh][idNodes[j]] < Integer.MAX_VALUE && distances[node][idNodes[j]] > (distances[neigh][idNodes[j]] + distances[node][neigh]))
									distances[node][idNodes[j]] = distances[neigh][idNodes[j]] + distances[node][neigh];
							}
						}
						else 
							queue1.add(neigh);
						visited.add(neigh);
						distances[neigh][node] = distance;
						distances[node][neigh] = distance;

					}
				}
			}
			
			distance++;
		}

		
	}
	
	public static void main(String[] args) {
		MyGraph g = new MyGraph("Teste");
		
		g.addEdge(0, 2);
		g.addEdge(1, 3);
		g.addEdge(1, 4);
		g.addEdge(2, 3);
		g.addEdge(4, 5);
		g.addEdge(5, 6);
		g.addEdge(6, 7);
		g.addEdge(7, 5);
		
		try {
			g.saveNodesFeaturesToFile("test");
		} catch(Exception e) {
			e.printStackTrace();
		}
		
		System.out.println(GlobalFunctions.getMaxFullyDisconnectedDegree(g));

	}	

				

}
