package network.graph;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Random;
import java.util.Scanner;
import java.util.Set;

import edu.uci.ics.jung.algorithms.importance.BetweennessCentrality;
import edu.uci.ics.jung.algorithms.scoring.PageRank;
import edu.uci.ics.jung.graph.UndirectedSparseGraph;
import main.DB;
import main.Run;
import main.RunConfig;
import network.functions.DistanceFunctions;
import network.functions.GlobalFunctions;
import network.functions.IndividualFunctions;

public class MyGraph {
	
	HashMap<Integer,EdgeList> edges = new HashMap<Integer,EdgeList>();
	
	HashSet<Edge> edgeSet = new HashSet<Edge>();
	
	HashMap<Integer,Node> nodes = new HashMap<Integer,Node>();
	
	HashMap<Integer,Integer> componentsSize = new HashMap<Integer,Integer>();
	
	public boolean COMPUTE_COMPONENTS_DYNAMICLY = false;
	
	DistanceFunctions distanceFunctions = new DistanceFunctions(this);
	
	String name;
	
	
	
	private int maxId = 0;
	private int numEdges = 0;
	private int time;
	
	UndirectedSparseGraph<Integer,Edge> gJung = new UndirectedSparseGraph<Integer,Edge>();
	BetweennessCentrality<Integer, Edge> bRanker;
	
	public MyGraph(String name) {
	
		this.name = name;
		bRanker = new BetweennessCentrality<Integer, Edge>(gJung);
		bRanker.setNormalizeRankings(true);
		bRanker.setRemoveRankScoresOnFinalize(false);
	

	}
	
	
	
	public String getName() {
		return name;
	}



	public UndirectedSparseGraph<Integer,Edge> getGraphJung() {
		return gJung;
	}
	
	public void clear() {
		this.nodes.clear();
		this.edges.clear();
		this.edgeSet.clear();
		this.componentsSize.clear();
		this.gJung = new UndirectedSparseGraph<Integer,Edge>();
		this.numEdges = 0;
	}

	
	
	public void removeDeadEdges(long deltaTime) {
		
		Iterator<Edge> iEdges = this.edgeSet.iterator();
		while(iEdges.hasNext()) {
			Edge e = iEdges.next();
			if(this.getTime() - e.timeWillDie > deltaTime) {
				this.removeEdge(e.getNodeSrc(), e.getNodeDest());
				this.removeEdge(e.getNodeDest(), e.getNodeSrc());
			}
		}
	}

	
	
	public int getNumEdges() {
		return numEdges;
	}

	public void setNumEdges(int numEdges) {
		this.numEdges = numEdges;
	}

	public int getMaxId() {
		return maxId;
	}
	
	

	public int getTime() {
		return time;
	}

	public void setTime(int time) {
		this.time = time;
	}

	public HashMap<Integer, Integer> getComponentsSize() {
		return componentsSize;
	}

	public int getComponentSize(int c) {
		int size = 0;
		try {
			size = this.componentsSize.get(c);
		} catch(Exception e) {
			System.out.println("component size " + c + " " + e.getMessage());
		}
		return size;
	}
	
	public void updateComponentSize(int c, int size) {
		this.componentsSize.put(c, size);
	}
	
	public int getNewComponentLabel() {
		HashSet<Integer> labels = new HashSet<Integer>(this.componentsSize.keySet());
		Random random = new Random();
		int label = random.nextInt();
		while(labels.contains(label))
			label = random.nextInt();
		return label;
	}
	
	public void mergeComponents(int c1, int c2) {
		this.componentsSize.put(c1, this.componentsSize.get(c1) + this.componentsSize.get(c2));
		this.componentsSize.remove(c2);
	}
	
	public boolean addNode(int nodeid) {
		if(!edges.containsKey(nodeid)) {
			edges.put(nodeid, new EdgeList(nodeid));
			Node node = new Node(nodeid);
			node.setComponentLabel(this.getNewComponentLabel());
			nodes.put(nodeid, node);
			componentsSize.put(node.getComponentLabel(), 1);
			
			if(nodeid > this.maxId)
				this.maxId = nodeid;
			
			gJung.addVertex(nodeid);
			
			return true;
		}
		return false;
		
	}
	
	public boolean isEmpty() {
		return this.nodes.size() == 0;
	}
	
	public int getNumNodes() {
		return this.nodes.size();
	}
	
	public Node getNode (int id) {
		return nodes.get(id);
	}
	
	public boolean removeNode(int node) {
		if(edges.containsKey(node)) {
			edges.remove(node);
			gJung.removeVertex(node);
			return true;
		}
		return false;		
	}
	
	public boolean addEdge(int node1, int node2) {
		boolean newEdge = this.addEdge(node1, new Edge(node1, node2, 1.0, (long)0));
		this.addEdge(node2, new Edge(node2, node1, 1.0, (long)0));
		return newEdge;
				
	}
	
	public void addEdge(int node1, int node2, double weight) {
		this.addEdge(node1, new Edge(node1, node2, weight, (long)0));
		this.addEdge(node2, new Edge(node2, node1, weight, (long)0));
				
	}

	public void addEdge(int node1, int node2, double weight, long timeWillDie) {
		this.addEdge(node1, new Edge(node1, node2, weight, timeWillDie));
		this.addEdge(node2, new Edge(node2, node1, weight, timeWillDie));
				
	}
	
	public boolean addEdge(int node1, Edge e) { //false if the edge already existed
		
		
		if(!edges.containsKey(node1)) {
			this.addNode(node1);
		}
		if(!edges.containsKey(e.getNodeDest())) {
			this.addNode(e.getNodeDest());
		}		
		
		if(this.COMPUTE_COMPONENTS_DYNAMICLY) {
			GlobalFunctions.updateComponentsEdgeAdded(this, node1, e.getNodeDest());
		}		
		
		if (!edges.get(node1).add(e)) { //if edge is old
			return false;
		}
		
		
		gJung.addEdge(e, node1, e.getNodeDest());
		edgeSet.add(e);
		
		this.numEdges++;
		return true;
		
	}	
	
	public void removeEdge(int node1, int node2) {
		if(this.edges.containsKey(node1)) {
			Edge e = this.edges.get(node1).remove(node2);
			this.numEdges--;
			
			gJung.removeEdge(e);
			edgeSet.remove(e);
			
		}
		if(this.edges.containsKey(node2)) {
			Edge e = this.edges.get(node2).remove(node1);
			this.numEdges--;
			
			gJung.removeEdge(e);
			edgeSet.remove(e);
		}
		
/*		if(this.getDegree(node1) == 0)
			this.removeNode(node1);
		
		if(this.getDegree(node2) == 0)
			this.removeNode(node2);*/
		
		if(this.COMPUTE_COMPONENTS_DYNAMICLY) {
			GlobalFunctions.updateComponentsEdgeRemoved(this, node1, node2);
		}			
		
	}
	
	public Edge removeRandomEdge() {
		Edge[] edgeArray = edgeSet.toArray(new Edge[0]);
		Random random = new Random();
		int r = random.nextInt(edgeArray.length);
		Edge e = edgeArray[r];
		this.removeEdge(e.nodeSrc, e.nodeDest);
		return e;
	}
		
	public Integer[] getNodes() {
		return this.edges.keySet().toArray(new Integer[0]);
	}
	
	public HashSet<Integer> getNodeSet() {
		return new HashSet<Integer>(this.edges.keySet());
	}
	
	public Edge[] getEdges(int nodeId) {
		if(!this.edges.containsKey(nodeId))
			return null;
		return this.edges.get(nodeId).toArray();
	}
	
	public Integer[] getNeighs(int nodeId) {
		return this.edges.get(nodeId).keySet().toArray(new Integer[0]);
	}	
	
	public Set<Integer> getNeighsSet(int nodeId) {
		
		//return new HashSet<Integer>(gJung.getNeighbors(nodeId));
		
		if(this.edges.containsKey(nodeId))
			return this.edges.get(nodeId).keySet();
		return new HashSet<Integer>();
	}		
	
	
	
	public HashSet<Edge> getEdgeSet() {
		return edgeSet;
	}
	

	public int getDegree(int nodeId) {
		if(this.edges.containsKey(nodeId))
			return this.edges.get(nodeId).size();
		return 0;
	}
	
	public boolean existsEdge(int node1, int node2) {
		if(edges.containsKey(node1) && edges.containsKey(node2))
			return this.edges.get(node1).containsKey(node2);
		return false;
	}
	
	public int aggregateGraph(MyGraph g) {
		Integer newNodes[] = g.getNodes();
		int numChanges = 0;
		for(int i=0; i<newNodes.length; i++) {
			if(!this.edges.containsKey(newNodes[i])) {
				this.addNode(newNodes[i]);
			}
			Edge newEdges[] = g.getEdges(newNodes[i]);
			for(int j=0; j<newEdges.length; j++) {
				numChanges += this.addEdge(newNodes[i], newEdges[j]) == true ? 1 : 0;
			}
		}
		return numChanges;
	}
	
	
	public String toString() {
		
		//return "" + edges;
		return this.nodes.size() + "," + this.numEdges;
	}
	
	public void loadGraphFromFile(String fileName) throws Exception {
	    FileReader fin = new FileReader(fileName);
	    Scanner src = new Scanner(fin);
	    
	    while(src.hasNext()) {
	    	int v1 = src.nextInt();
	    	int v2 = src.nextInt();
	    	this.addEdge(v1, v2);
	    	src.nextLine();
	    }
	    
	    src.close();
	    fin.close();

	}
	
	public void saveGraphToFile(String filename) throws Exception {
		
		if(!this.isEmpty()) {
		
			File file = new File(DB.getGraphsPath() + filename);
			FileWriter writer = new FileWriter(file);
			PrintWriter out = new PrintWriter(writer);  
	
			Iterator<Integer> inodes = this.nodes.keySet().iterator();
			while(inodes.hasNext()) {
				int nodeid = inodes.next();
				ArrayList<Edge> elist = edges.get(nodeid).getEdgesL();
				for(int j=0; j<elist.size(); j++) {
					if(nodeid<elist.get(j).getNodeDest())
						out.println(nodeid + " " + elist.get(j).getNodeDest() + " " + elist.get(j).getWeight());
				}
			}
			
			out.close();
			writer.close();
		}
		
		
	}
	
	public void saveNodesFeaturesToFile(String filename) throws Exception {
		
		double closeness = 0, betweenness = 0, pageRank = 0, cc = 0;
		
		
		File file = new File(filename);
		FileWriter writer = new FileWriter(file);
		PrintWriter out = new PrintWriter(writer);  

		distanceFunctions.generateDistances(); 
		
		
		
		BetweennessCentrality<Integer, Edge> bRanker = new BetweennessCentrality<Integer, Edge>(gJung);
		
		if(RunConfig.NF_RECORD_BETWEENNESS) {
			bRanker.setRemoveRankScoresOnFinalize(false);
			bRanker.evaluate();
		}
		
		PageRank<Integer, Edge> pRanker = new PageRank<Integer, Edge>(gJung, 0.1);

		if(RunConfig.NF_RECORD_PAGERANK) {
			pRanker.evaluate();
		}
		 
	
		
		//out.println("* " + distanceFunctions.getAvgDistance() + " " + distanceFunctions.getMaxDistance());
		
		Iterator<Integer> inodes = this.nodes.keySet().iterator();
		while(inodes.hasNext()) {
			int nodeid = inodes.next();
			out.print(nodeid + " ");
			
			if(RunConfig.NF_RECORD_CLOSENESS) {
				closeness = distanceFunctions.getNodeCloseness(nodeid);
			}
			
			
			out.print(closeness);
			out.print(" " + this.getDegree(nodeid));

			if(RunConfig.NF_RECORD_BETWEENNESS) {

				int n = this.nodes.size();
				double normalization = (n-1)*(n-2)/2;
				try {
					betweenness = bRanker.getVertexRankScore(nodeid)/normalization;
				} catch(Exception e) {
					betweenness = 0;
				}
			}
			out.print(" " + betweenness);

			if(RunConfig.NF_RECORD_PAGERANK) {
				pageRank = pRanker.getVertexScore(nodeid);
			}
			out.print(" " + pageRank);
			
			if(RunConfig.NF_RECORD_CC) {
				cc = IndividualFunctions.clusteringCoefficient(this, nodeid);
			}

			out.print(" " + cc);
			
			out.println();
		}
		
		out.close();
		writer.close();
		
		
	}
		
	
	public void saveEdgesFeaturesToFile(String filename) throws Exception {
		
		if(!this.isEmpty()) {
			

			File file = new File(DB.getGraphsPath() + filename);
			FileWriter writer = new FileWriter(file);
			PrintWriter out = new PrintWriter(writer);  
		
			int n = this.nodes.size();
			double normalization = (n-1)*(n-2)/2;
			//bRanker.evaluate();
			
		 
			Iterator<Integer> inodes = this.nodes.keySet().iterator();
			while(inodes.hasNext()) {
				int nodeid = inodes.next();
				ArrayList<Edge> elist = edges.get(nodeid).getEdgesL();
				for(int j=0; j<elist.size(); j++) {
					if(nodeid<elist.get(j).getNodeDest()) {
						Edge edge = elist.get(j);
						
	/*					if(edge.getNodeSrc() == 1 && edge.getNodeDest() == 39)
							System.out.println("stop");*/
						DecimalFormat df = new DecimalFormat("0.00");
						
						//double totalTS = Math.ceil((double)this.getTime()/NetworkAnalyzer.AGGREGATE_GRAPH_INTERVAL);
						double totalTS = Math.ceil((double)this.getTime())-1; //-1 because it goes here after the first event of the next timestep!
						double edgePersistency = Math.min(edge.getNumTimeSteps()/totalTS, 1.0);
						double betweeness = 0;
						int to = IndividualFunctions.getTopologicalOverlap(this, edge);
						int degreeSrc = this.getDegree(edge.getNodeSrc());
						int degreeDest = this.getDegree(edge.getNodeDest());
	/*					try {
							betweeness = bRanker.getEdgeRankScore(edge)/normalization;
								
						} catch(java.lang.RuntimeException e) { //could not compute the betweeness for an edge
							//System.out.println("Could not compute the edge betweeness of edge " + edge + "\n" + e.getMessage());
						}*/
						/*out.println(nodeid + " " + edge.getNodeDest() + " " + betweeness +
								" " + edgePersistency + " " + edge.getWeight() + " " +
								to + " " + degreeSrc + " " + degreeDest);*/
						try {
							out.println(nodeid + " " + edge.getNodeDest() + " " + df.format(edgePersistency) + " " + df.format(edge.getWeight()) + " " +
									df.format((double)to/( (degreeSrc-1) + (degreeDest-1) - to)) + " " + to + " " + degreeSrc + " " + degreeDest+ " " + edge.getCount());
						} catch(ArithmeticException e) { //division by 0: degrees are 1
							out.println(nodeid + " " + edge.getNodeDest() + " " + df.format(edgePersistency) + " " + df.format(edge.getWeight()) + " " +
									0 + " " + to + " " + degreeSrc + " " + degreeDest + " " + edge.getCount());

						}
					}
				}
			}
			
			out.close();
			writer.close();		
		}
		

	}
	

}
