package propagation;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Random;
import java.util.Scanner;

import loganalysis.EdgesClassReader;
import main.DB;
import main.RunConfig;

import org.apache.commons.collections15.keyvalue.MultiKey;
import org.apache.commons.math.stat.descriptive.DescriptiveStatistics;

import network.functions.IndividualFunctions;
import network.graph.Edge;
import network.graph.MyGraph;

public class PropagationByClassAnalyzer {
	
	
	HashSet<Integer> infected = new HashSet<Integer>();
	MyGraph g; //event graph
	MyGraph G; //cumulative graph
	HashMap<Integer,Double> delays = new HashMap<Integer,Double>();
	int countClassesTestSet[] = new int[DB.NUM_CLASSES+2];
	//class 0 = non-classified edges
	//class n+1 = non-classified edges that would be classified as 3
	
	boolean constructG = true;
	int countNewEdgesHighTO = 0;
	int countNewEdgesLowTO = 0;
	HashSet<MultiKey<Integer>> newEdgesHighTO = new HashSet<MultiKey<Integer>>();
	
	
	HashMap<Integer,ArrayList<Integer>> paths = new HashMap<Integer,ArrayList<Integer>>();
	
	int firstNode = 0;
	double timeToInfect = 0.01;
	int totalEncounters = 0;
	
	
	int totalPeople;
	
	double time, timeFirstInfection, timeLastInfection;
	
	EdgesClassReader eclasses;
	
	int maxnode = -1;
	
	PrintWriter outInfection;
	
	public PropagationByClassAnalyzer(MyGraph g, int n) {
		this.totalPeople = n;
		this.g = g;
		this.eclasses = new EdgesClassReader();
		
		String fileName = DB.getGraphsPath() + "infection" + DB.getDBName() + "_class0_threshold" + RunConfig.P_RANDOM + "_start" + DB.getTrainingSetUpperBound() + "_end" + DB.getTestSetUpperBound() + ".dat";
		File file = new File(fileName);
		try {
			FileWriter writer = new FileWriter(file);
			outInfection = new PrintWriter(writer);
		} catch(Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		
		for(int i=0; i<DB.NUM_CLASSES+2; i++)
			this.countClassesTestSet[i] = 0;

		if(this.constructG)
			this.loadAggregatedGraph();
	}
	
	public void loadTopologicalOverlaps(File file) {
		try {
			FileReader fin = new FileReader(file);
			Scanner src = new Scanner(fin);
			
			while(src.hasNext()) {
				int v1 = src.nextInt();
				int v2 = src.nextInt();
				double to = src.nextDouble();
				
				if(to >= eclasses.getMinTONonRandomValue()) {
					newEdgesHighTO.add(new MultiKey<Integer>(v1, v2));
					newEdgesHighTO.add(new MultiKey<Integer>(v2, v1));
				}
				
			}
			
			src.close();
			fin.close();
		}catch(Exception e) {
			
		}

	}
	
	
	
	public void loadAggregatedGraph() {
		G = new MyGraph("ClassPropagationG");
		try {
			G.loadGraphFromFile(DB.getGraphsPath() + "Aggregated24-" + DB.getTrainingSetUpperBound() + ".graph");

			
			System.out.println("reading which non-existing edges have a high TO...");
			
			
			//save topological overlaps to use later
		    	
		 	String fileName = DB.getGraphsPath() + "topct" + DB.getDBName() + "_class0_threshold0.1_start" + DB.getTrainingSetUpperBound() + ".dat";
			File file = new File(fileName);
			if(file.exists()) {
				this.loadTopologicalOverlaps(file);
			}
			else {
				FileWriter writer = new FileWriter(file);
				PrintWriter out = new PrintWriter(writer);
				
				
				Integer nodes[] = G.getNodes();
				for(int v1=0; v1<nodes.length; v1++) {
					System.out.println(v1);
					for(int v2=v1+1; v2<nodes.length; v2++) {
						if(!G.existsEdge(nodes[v1], nodes[v2])) {
							double to = IndividualFunctions.getTopologicalOverlapPct(G, nodes[v1], nodes[v2]);
							if (to >= eclasses.getMinTONonRandomValue()) {
						
								newEdgesHighTO.add(new MultiKey<Integer>(nodes[v1], nodes[v2]));
								newEdgesHighTO.add(new MultiKey<Integer>(nodes[v2], nodes[v1]));
								out.println(nodes[v1] + " " + nodes[v2] + " " + to);
							}
							                 
						}
					}
				}
				
				
			
				out.close();
				writer.close();
			}
			System.out.println("done!");
			
		} catch(Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}
	
	public void finalize() {
		this.outInfection.close();
		this.saveClassCount();
	}
	
	public void saveClassCount() {
		String fileName = DB.getGraphsPath() + "countClasses" + DB.getDBName() + "_class0_threshold" + RunConfig.P_RANDOM + "_start" + DB.getTrainingSetUpperBound() + "_end" + DB.getTestSetUpperBound() + ".dat";
		File file = new File(fileName);
		try {
			FileWriter writer = new FileWriter(file);
			PrintWriter out = new PrintWriter(writer);
			for(int i=0; i<=DB.NUM_CLASSES; i++) {
				double pclass = (double)this.countClassesTestSet[i]/this.totalEncounters;
				double pEdgeGivenClass = pclass / this.eclasses.getNumEdgesPerClass(i);
				out.println(i + " " + this.countClassesTestSet[i] + " " + pclass + " " + eclasses.getExpectedEncountersByClass(i) + " " + pEdgeGivenClass);
			}
			out.println((DB.NUM_CLASSES+1) + " " + this.countClassesTestSet[(DB.NUM_CLASSES+1)] + " " + (double)this.countClassesTestSet[(DB.NUM_CLASSES+1)]/this.countClassesTestSet[0] + " " + (double)this.countNewEdgesLowTO/(this.eclasses.getNumEdgesPerClass(0)-(this.newEdgesHighTO.size()/2.0)) + " " + (double)this.countNewEdgesHighTO/(this.newEdgesHighTO.size()/2.0));

			out.close();
			writer.close();
		} catch(Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}
	
	public void infectFirstNode(int v) throws EveryoneInfectedException {
		this.firstNode = v;
		this.infect(v,-1);
	}
	
	public void infectRandomNode() throws EveryoneInfectedException {
        Random r = new Random(2);
        int randomNode = r.nextInt(this.totalPeople);
        System.out.println("FIRST NODE INFECTED: " + randomNode);
        this.firstNode = randomNode;
        infect(this.firstNode,-1);
	}
	
	private void markInfected(int v, int vsource) {
		this.infected.add(v);
		System.out.println("time: " + this.time + ", node " + v + " infected.");
		if(vsource >= 0) {
			if(!this.paths.containsKey(v)) {
				this.paths.put(v, new ArrayList<Integer>(vsource));
				if(this.paths.containsKey(vsource)) {
					this.paths.get(v).addAll(this.paths.get(vsource));
				}
			}
			
			if(this.infected.size() == 2)
				this.timeFirstInfection = this.time;
			
			double timeInfection = this.time - this.timeFirstInfection;
			delays.put(v,this.time - this.timeFirstInfection);
			outInfection.println(timeInfection + " " + v + " " + vsource + " " + this.eclasses.getEdgeClass(v, vsource));
		}
			
	}
	
	public void infect(int v, int vsource) throws EveryoneInfectedException {
		
	
		if(!this.infected.contains(v)) {
			
			markInfected(v,vsource);
			
			Edge edges[] = g.getEdges(v);
			if(edges != null) {
				this.time = this.time + this.timeToInfect;
				for(int i=0; i<edges.length; i++) {
					if(!this.infected.contains(edges[i].getNodeDest()))
						this.infect(edges[i].getNodeDest(),v);
				}
			}
			
			

			
			if(this.infected.size() == this.totalPeople) {
				this.timeLastInfection = this.time;
				System.out.println("*******Everyone infected! total time: " + (this.timeLastInfection - this.timeFirstInfection)
									+ "\n last node infected: " + v);
				throw new EveryoneInfectedException();
			}

		}
			
	}
	
	public double getInfectedPct() {
		return (double)(infected.size())/totalPeople;
	}
	
	public void printDelays() {
		try {
			File file = new File(DB.getFeaturesPath() + "/propagation/results" + RunConfig.RANDOM_EDGES_USE + "-" + RunConfig.RICH_CLUB_USE + "/propagation" + this.firstNode);
			FileWriter writer = new FileWriter(file);
			PrintWriter out = new PrintWriter(writer);
			
			Integer nodes[] = infected.toArray(new Integer[0]);
			
			for(int i=0; i<nodes.length; i++) 
				out.println(nodes[i] + " " + delays.get(nodes[i]));
			
			out.close();
			writer.close();
			
		} catch(Exception e) {
			System.out.println(e.getMessage());
		}
	}
	
	public void processContact(int v1, int v2, long time) throws EveryoneInfectedException {
		this.time = time;
		
		this.totalEncounters++;
		


		if(infected.contains(v1) || this.infected.size() == 0) {
			infect(v2,v1);
		}
		else if(infected.contains(v2)) {
			infect(v1,v2);
		}
		
		if(this.constructG) {
			if(G.addEdge(v1, v2)) {
				if(IndividualFunctions.getTopologicalOverlapPct(G, v1, v2) >= eclasses.getMinTONonRandomValue()) {
					this.countNewEdgesHighTO++;
					this.newEdgesHighTO.add(new MultiKey<Integer>(v1, v2));
					this.newEdgesHighTO.add(new MultiKey<Integer>(v2, v1));
				}
				else
					this.countNewEdgesLowTO++;
			}
		}
		
		int eclass = this.eclasses.getEdgeClass(v1, v2);
		if(eclass == 0 && IndividualFunctions.getTopologicalOverlapPct(G, v1, v2) >= eclasses.getMinTONonRandomValue())
			this.countClassesTestSet[DB.NUM_CLASSES+1]++;
		this.countClassesTestSet[eclass]++;		
	}
	
	

}
