package propagation;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Random;
import java.util.Scanner;

import loganalysis.EdgesClassReader;
import main.DB;
import main.RunConfig;

import org.apache.commons.collections15.keyvalue.MultiKey;
import org.apache.commons.math.stat.descriptive.DescriptiveStatistics;


//import network.functions.GlobalFunctions;
import network.functions.IndividualFunctions;
import network.graph.Edge;
import network.graph.MyGraph;

public class PropagationByClassBroadcast extends PropagationSNA{
	
	public final static int ALL_CLASSES = 0;
	public final static int NO_CLASS = 0;
	
	HashSet<Integer> infected = new HashSet<Integer>();
	HashSet<Integer> nodesAround = new HashSet<Integer>();
	HashSet<Integer> nodesNewEdges = new HashSet<Integer>();
	MyGraph G = new MyGraph("propG"); //cumulative graph
	HashMap<Integer,Double> delays = new HashMap<Integer,Double>();
	DescriptiveStatistics dsDelays = new DescriptiveStatistics();
	int countClassesTestSet[] = new int[DB.NUM_CLASSES+2];
	//class 0 = non-classified edges
	//class n+1 = non-classified edges that would be classified as 3
	
	HashMap<Integer,Integer> infectionPerHour = new HashMap<Integer,Integer>();
	HashMap<Integer,Integer> msgsPerHour = new HashMap<Integer,Integer>();
	HashMap<Integer,Double> tputPerHour = new HashMap<Integer,Double>();
	
	boolean constructG = false;
	int countNewEdgesHighTO = 0;
	int countNewEdgesLowTO = 0;
	HashSet<MultiKey<Integer>> newEdgesHighTO = new HashSet<MultiKey<Integer>>();
	
	
	HashMap<Integer,ArrayList<Integer>> paths = new HashMap<Integer,ArrayList<Integer>>();
	
	int firstNode = 0;
	double timeToInfect = 0.01;
	int totalEncounters = 0, totalTransmissions = 0;
	
	//int allowedClass = -1;
	//int forbiddenClass = -1;
	

	double time, timeFirstInfection, timeLastInfection, TO;
	
	EdgesClassReader eclasses;
	
	int maxnode = -1;
	
	PrintWriter outInfection;
	
	public PropagationByClassBroadcast() {
		
		if(DB.myDB == 2)
			return;
		
		System.out.println("time first infection:" + RunConfig.START_TIME_INFECTION);
		//this.forbiddenClass = RunConfig.FORBIDDEN_CLASS;
		//this.allowedClass = RunConfig.ALLOWED_CLASS;
		//this.g = g;
		this.eclasses = new EdgesClassReader();
		
		String fileName = DB.getGraphsPath() + "infection" + DB.getDBName() + "_class0_threshold" + RunConfig.P_RANDOM + "_start" + DB.getTrainingSetUpperBound() + "_end" + DB.getTestSetUpperBound() + ".dat";
		File file = new File(fileName);
		try {
			FileWriter writer = new FileWriter(file);
			outInfection = new PrintWriter(writer);
		} catch(Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		
		for(int i=0; i<DB.NUM_CLASSES+2; i++)
			this.countClassesTestSet[i] = 0;

		if(this.constructG)
			this.loadAggregatedGraph();
	}
	
	public void loadTopologicalOverlaps(File file) {
		try {
			FileReader fin = new FileReader(file);
			Scanner src = new Scanner(fin);
			
			while(src.hasNext()) {
				int v1 = src.nextInt();
				int v2 = src.nextInt();
				double to = src.nextDouble();
				
				if(to >= eclasses.getMinTONonRandomValue()) {
					newEdgesHighTO.add(new MultiKey<Integer>(v1, v2));
					newEdgesHighTO.add(new MultiKey<Integer>(v2, v1));
				}
				
			}
			
			src.close();
			fin.close();
		}catch(Exception e) {
			
		}

	}
	
	
	
	public void loadAggregatedGraph() {
		G = new MyGraph("ClassPropagationG");
		try {
			G.loadGraphFromFile(DB.getGraphsPath() + "Aggregated24-" + DB.getTrainingSetUpperBound() + ".graph");

			
			System.out.println("reading which non-existing edges have a high TO...");
			
			
			//save topological overlaps to use later
		    	
		 	String fileName = DB.getGraphsPath() + "topct" + DB.getDBName() + "_class0_threshold0.1_start" + DB.getTrainingSetUpperBound() + ".dat";
			File file = new File(fileName);
			if(file.exists()) {
				this.loadTopologicalOverlaps(file);
			}
			else {
				FileWriter writer = new FileWriter(file);
				PrintWriter out = new PrintWriter(writer);
				
				
				Integer nodes[] = G.getNodes();
				for(int v1=0; v1<nodes.length; v1++) {
					System.out.println(v1);
					for(int v2=v1+1; v2<nodes.length; v2++) {
						if(!G.existsEdge(nodes[v1], nodes[v2])) {
							double to = IndividualFunctions.getTopologicalOverlapPct(G, nodes[v1], nodes[v2]);
							if (to >= eclasses.getMinTONonRandomValue()) {
						
								newEdgesHighTO.add(new MultiKey<Integer>(nodes[v1], nodes[v2]));
								newEdgesHighTO.add(new MultiKey<Integer>(nodes[v2], nodes[v1]));
								out.println(nodes[v1] + " " + nodes[v2] + " " + to);
							}
							                 
						}
					}
				}
				
				
			
				out.close();
				writer.close();
			}
			System.out.println("done!");
			
		} catch(Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}
	
	public void finalizePropagation() {
		this.outInfection.close();
		if(RunConfig.SAVE_PROPAGATION_LOGS) {
			this.saveClassCount();
		}
		this.printDelays();
		//System.out.println("num components: "+ GlobalFunctions.getNumberOfComponents(G));
		
	}
	
	public void saveClassCount() {
		String fileName = DB.getGraphsPath() + "countClasses" + DB.getDBName() + "_class0_threshold" + RunConfig.P_RANDOM + "_start" + DB.getTrainingSetUpperBound() + "_end" + DB.getTestSetUpperBound() + ".dat";
		File file = new File(fileName);
		try {
			FileWriter writer = new FileWriter(file);
			PrintWriter out = new PrintWriter(writer);
			for(int i=0; i<=DB.NUM_CLASSES; i++) {
				double pclass = (double)this.countClassesTestSet[i]/this.totalEncounters;
				double avgEdgeGivenClass = (double)this.countClassesTestSet[i] / this.eclasses.getNumEdgesPerClass(i);
				out.println(i + " " + this.countClassesTestSet[i] + " " + pclass + " " + eclasses.getExpectedEncountersByClass(i) + " " + avgEdgeGivenClass);
			}
			out.println((DB.NUM_CLASSES+1) + " " + this.countClassesTestSet[(DB.NUM_CLASSES+1)] + " " + (double)this.countClassesTestSet[(DB.NUM_CLASSES+1)]/this.countClassesTestSet[0] + " " + (double)this.countNewEdgesLowTO/(this.eclasses.getNumEdgesPerClass(0)-(this.newEdgesHighTO.size()/2.0)) + " " + (double)this.countNewEdgesHighTO/(this.newEdgesHighTO.size()/2.0));

			out.close();
			writer.close();
		} catch(Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}
	
	public void infectFirstNode(int v) throws EveryoneInfectedException {
		this.firstNode = v;
		this.transmitInfection(v,-1);
	}
	
	
	private void markInfected(int v, int vsource) {
		if(this.infected.isEmpty())
			this.firstNode = v;
		this.infected.add(v);
		
		//System.out.println("time: " + this.time + ", node " + v + " infected.");
		if(vsource >= 0) {
			if(!this.paths.containsKey(v)) {
				this.paths.put(v, new ArrayList<Integer>(vsource));
				if(this.paths.containsKey(vsource)) {
					this.paths.get(v).addAll(this.paths.get(vsource));
				}
			}
			
			if(this.infected.size() == 2)
				this.timeFirstInfection = this.time;
			
			double timeInfection = this.time - this.timeFirstInfection;
			delays.put(v,this.time - this.timeFirstInfection);
			dsDelays.addValue(this.time - this.timeFirstInfection);
			outInfection.println(timeInfection + " " + v + " " + vsource + " " + this.eclasses.getEdgeClass(v, vsource));
		}
			
	}
	
	//infect all neighbors
	private void infectNeighbors(int v) throws EveryoneInfectedException {
		Edge edges[] = g.getEdges(v);
		if(edges != null) {
			this.time = this.time + this.timeToInfect;
			for(int i=0; i<edges.length; i++) {
				int vDest = edges[i].getNodeDest();
				int eclass = this.eclasses.getEdgeClass(v, vDest);
				if(eclass !=0 && !this.infected.contains(vDest)) {
					markInfected(vDest,v);
					propagateInfection(vDest);
				}
			}
		}
	}
	
	//propagate the infection through neighbors when allowed
	private void propagateInfection(int v) throws EveryoneInfectedException {
		Edge edges[] = g.getEdges(v);
		if(edges != null) {
			this.time = this.time + this.timeToInfect;
			for(int i=0; i<edges.length; i++) {
				if(this.allowInfection(edges[i].getNodeDest(),v, (long)time) && !this.infected.contains(edges[i].getNodeDest()))
					this.transmitInfection(edges[i].getNodeDest(),v);
			}
		}
	}	
	
/*	public void infect(int v, int vsource) throws EveryoneInfectedException {
		
	
		if(!this.infected.contains(v)) {
			
			markInfected(v,vsource);
			if(RunConfig.BROADCAST_INFECTION)
				this.infectNeighbors(vsource);
			
			Edge edges[] = g.getEdges(v);
			if(edges != null) {
				this.time = this.time + this.timeToInfect;
				for(int i=0; i<edges.length; i++) {
					if(!this.infected.contains(edges[i].getNodeDest()))
						this.infect(edges[i].getNodeDest(),v);
				}
			}
			

		}
			
	}*/
	
	public void transmitInfection(int v, int vsource) throws EveryoneInfectedException {
		
		
		if(!this.infected.contains(v)) {
			
			totalTransmissions++;
			markInfected(v,vsource);
			//if(RunConfig.BROADCAST_INFECTION)
				this.infectNeighbors(vsource);
			
			this.propagateInfection(v);
			

		}
			
	}	
	
	private void printDelays() {
		try {
			
			double nodesAroundSize = nodesAround.size();
			String result = this.timeFirstInfection + " " + this.firstNode + " " + (double)this.infected.size()/(double)this.nodesAround.size() + " " + this.dsDelays.getMean() + " " +
			this.dsDelays.getPercentile(50) + " " + this.dsDelays.getStandardDeviation() + " " + this.totalTransmissions;
			System.out.println(result);
			this.nodesAround.retainAll(nodesNewEdges);
			System.out.println(nodesAround.size()/nodesAroundSize);
			
			//general result
			if(RunConfig.SAVE_PROPAGATION_LOGS) {
				String fileName = DB.getGraphsPath() + "propagation" + DB.getDBName() + "_fbd" + ClassesAllowed.getForbiddenClasses() + "_allw" + ClassesAllowed.getAllowedClasses() + "_threshold" + RunConfig.P_RANDOM + "_start" + DB.getTrainingSetUpperBound() + "_end" + DB.getTestSetUpperBound() + ".dat";
				System.out.println(fileName);
				File file = new File(fileName);
				FileWriter writer = new FileWriter(file, true);
				PrintWriter out = new PrintWriter(writer);
				
	
				out.println(result);
				
				out.close();
				writer.close();
				//------------------------------------------
				//result per hour
				fileName = DB.getGraphsPath() + "propagation/propagationPerHour" + DB.getDBName() + "_fbd" + ClassesAllowed.getForbiddenClasses() + "_allw" + ClassesAllowed.getAllowedClasses() + "_threshold" + RunConfig.P_RANDOM + "_start" + this.timeFirstInfection + ".dat";
				file = new File(fileName);
				writer = new FileWriter(file);
				out = new PrintWriter(writer);
				
				Iterator<Integer> hoursIt = infectionPerHour.keySet().iterator();
				while(hoursIt.hasNext()) {
					int hour = hoursIt.next();
					out.println(hour + " " + infectionPerHour.get(hour)/nodesAroundSize + " " + this.msgsPerHour.get(hour) + " " + this.tputPerHour.get(hour));
				}
				
				out.close();
				writer.close();
			}
			
		} catch(Exception e) {
			System.out.println(e.getMessage());
		}
	}
	
	
	private boolean allowInfection(int v1, int v2, long time) {
		
		//System.out.println(this.g.getTime());
		
		if(this.totalEncounters == 0) {
			//select a random time to start the infection
			this.TO = time;
			int maxTime = (int)(DB.getInfectionDuration());
			
			if(RunConfig.START_TIME_INFECTION <= 0) {
				Random random = new Random();
				this.timeFirstInfection = this.TO + random.nextInt(maxTime);
				RunConfig.START_TIME_INFECTION = this.timeFirstInfection;
			}
			else
				this.timeFirstInfection = RunConfig.START_TIME_INFECTION;
			
			this.timeLastInfection = this.timeFirstInfection + DB.getInfectionDuration();
		}
		

		if(time >= this.timeFirstInfection && time <= this.timeLastInfection) {
			
			//System.out.print('*');
			
			int numHours = (int)Math.floor((time - this.timeFirstInfection)/3600.0);
			if(!this.infectionPerHour.containsKey(numHours)) {
				this.infectionPerHour.put(numHours, this.infected.size());
				this.msgsPerHour.put(numHours, this.totalTransmissions);
				this.tputPerHour.put(numHours, this.totalTransmissions/(double)(this.infected.size()));
			}
			
			
			int eclass = this.eclasses.getEdgeClass(v1, v2);
			
			if(eclass == 0) { //new edge
				nodesNewEdges.add(v1);
				nodesNewEdges.add(v2);
				return false;
			}
			    
			
		
			this.totalEncounters++;
			
			
			
			this.nodesAround.add(v1);
			this.nodesAround.add(v2);
			
			this.countClassesTestSet[eclass]++;	
			
			if(ClassesAllowed.isAllowed(eclass))
				return true;
			
		//	if( (allowedClass == eclass || allowedClass == ALL_CLASSES) && (forbiddenClass != eclass || forbiddenClass == NO_CLASS))  
		//		return true;
			
		}
			
		return false;

		

	}
	
	public void processContact(int v1, int v2, long time) throws EveryoneInfectedException {
		

		
		this.time = time;
		

		if(allowInfection(v1, v2, time)) {
		
			if(infected.contains(v1) || this.infected.isEmpty()) {
				transmitInfection(v2,v1);
			}
			else if(infected.contains(v2)) {
				transmitInfection(v1,v2);
			}
		}
		
		
		if(this.constructG) {
			int eclass = this.eclasses.getEdgeClass(v1, v2);
			if(eclass != 0) {
				if(G.addEdge(v1, v2)) {
					if(IndividualFunctions.getTopologicalOverlapPct(G, v1, v2) >= eclasses.getMinTONonRandomValue()) {
						this.countNewEdgesHighTO++;
						this.newEdgesHighTO.add(new MultiKey<Integer>(v1, v2));
						this.newEdgesHighTO.add(new MultiKey<Integer>(v2, v1));
					}
					else
						this.countNewEdgesLowTO++;
				}
				
				if(eclass == 0 && IndividualFunctions.getTopologicalOverlapPct(G, v1, v2) >= eclasses.getMinTONonRandomValue())
					this.countClassesTestSet[DB.NUM_CLASSES+1]++;
			}
					
		}
	}
	
	

}
