package scheduler;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Scanner;

import main.DB;
import network.graph.Edge;

public class SchedulerSeveralFiles extends Scheduler {
	
    
    //Trace files available
    private final static Integer BUFFER_SIZE = 10; //number of lines I can read each time of each file
    private Integer currentLine = 0;
    File folder;
 
    public SchedulerSeveralFiles(String fileName) throws IOException  {
    	super(fileName);
    	this.folder = new File(fileName);
	    loadContactTrace();
	}

	public void loadContactTrace() throws IOException {
	    Integer id1, id2;
	    Long tIni;
	    HashSet<Integer> newEdges = new HashSet<Integer>();
	    HashSet<Integer> previousEdges = new HashSet<Integer>();;
	    
	    File[] listOfFiles = this.folder.listFiles();

	    for (int i=0; i<listOfFiles.length; i++) {
	    	//System.out.println(i + "/" + listOfFiles.length);

	    	if (listOfFiles[i].isFile()) {

	    		FileReader fin = new FileReader(listOfFiles[i].getAbsolutePath());
	    		Scanner src = new Scanner(fin);
	    		
	    		String fileName = listOfFiles[i].getName();
	    		id1 = Integer.parseInt(fileName.substring(0, fileName.indexOf('.')));
	    		
	    		//keeps track of the first connection of every edge
	    		HashMap<Integer,Long> edgeTime = new HashMap<Integer,Long>();
	    		
	    		previousEdges.clear();
	    		
	    		//go to the current line
	    		int count = 0;
	    		while(src.hasNext() && count < this.currentLine ) {
	    			src.nextLine();
	    			count++;
	    		}
	    		
	    		count = 1;
	    	    while (src.hasNext() && count < SchedulerSeveralFiles.BUFFER_SIZE ) {
	    	    	count++;	    
	    	    	tIni = src.nextLong()+1;
	    	    	//if(tIni == 0 && id1 == 30)
	    	    		//System.out.println("*******" + id1 + ":" + tIni);
	    	    	String line = src.nextLine();
	    	    	Scanner edges = new Scanner(line);
	    	    	while(edges.hasNext()) {
	    	    		id2 = edges.nextInt();
	    	    		//if(id2 == 61)
	    	    			//System.out.println(id2);
	    	    		newEdges.add(id2);
	    	    		
	    	    		
	    	    		if(!previousEdges.contains(id2)) {
	    	    			super.addConnection(tIni, id1, id2, 0.0);
	    	    			edgeTime.put(id2, tIni);
	    	    		}
	    	    		else {
	    	    			//update the edge
	    	    			Event e = super.events.get(edgeTime.get(id2));
	    	    			NodeEvents ne = e.getNodeEvents(id1);
	    	    			Edge edge = ne.cList.get(id2);
	    	    			//edge.incrementEdge(new Edge(id1, id2, 0.0, tIni+1));
	    	    			previousEdges.remove(id2);
	    	    		}
	    	    	}
	    	    	Iterator<Integer> pEdges = previousEdges.iterator();
	    	    	while(pEdges.hasNext()) { 
	    	    		super.addDisconnection(tIni, id1, pEdges.next());
	    	    	}
	    	    	previousEdges = new HashSet<Integer>(newEdges);
	    	    	newEdges.clear();
	    	    }	  
	    	    fin.close();
    		}
	    	
	    }
	    this.currentLine += SchedulerSeveralFiles.BUFFER_SIZE;
	}
	
	public Event getNextEvent() throws Exception {
		if(schedule.peek().getTime() >= this.currentLine)
			loadContactTrace();
		return super.getNextEvent();
	}
	
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		
		try {
			SchedulerSeveralFiles s = new SchedulerSeveralFiles(DB.getFilePath());
			while(s.hasNextEvent())
				System.out.println(s.getNextEvent());
		} catch(Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}

	}

}
