package scheduler;

import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.PriorityQueue;
import java.util.Scanner;

import main.DB;
import main.RunConfig;

public class SchedulerTestSet extends Scheduler {
	
    FileReader fin;
    Scanner src;	
    HashSet<Integer> activeNodes = new HashSet<Integer>();
    
    //Trace files available
    private final Integer BUFFER_SIZE = 50000;

    public SchedulerTestSet(String fileName) throws IOException  {
    	super(fileName);
	    fin = new FileReader(fileName);
	    src = new Scanner(fin);
	    loadContactTrace();
	}
    

	public HashSet<Integer> getActiveNodes() {
		return activeNodes;
	}



	public void loadContactTrace() throws IOException {
	    int id1, id2;
	    Long tIni, tFin;

	    //System.out.print("loading...");
	    
	    int count = 1;
	    while (src.hasNext() && count < BUFFER_SIZE) {
	    	count++;
	    	
	    	id1 = src.nextInt();
	    	id2 = src.nextInt(); 
	    	
	    	activeNodes.add(id1);
	    	activeNodes.add(id2);
	    		
	    	tIni = src.nextLong();
	    	tFin = src.nextLong();
	    	Double weight = (double)(tFin-tIni)/DB.getWeightScale();
	    	
	    	//System.out.println(id1 + " " + id2 + " " + tIni + " " + tFin);
	    	
	    	super.addConnection(tIni, id1, id2, weight);
	    	super.addDisconnection(tFin, id1, id2);
	    	
	    	try {
	    		src.nextLine();
	    	} catch(java.util.NoSuchElementException nsee) {
	    		System.out.println("End of file...");
	    	}
	    }
	    
	    //System.out.println("done!");
	    
	    if(!src.hasNext()) {
	    	//src.close();
	    	fin.close();
	    }

    
	}
	
	public Event getNextEvent() throws Exception {
		if(schedule.size() < BUFFER_SIZE/10 && src.hasNext())
			loadContactTrace();
		return super.getNextEvent();
	}
	

}
